--[[
File: Interface.lua
Author: Zormus
--]]

setfenv(1, select(2, ...))

constants.minWidth = 100
constants.maxWidth = 260
constants.defaultWidth = 160

constants.minSpacing = 28
constants.maxSpacing = 36
constants.defaultSpacing = 34

constants.minFade = 0
constants.maxFade = 75
constants.defaultFade = 50

local context = Interface.Create("Context", "Menu")
context:SetSecureMode("restricted")

menu = Interface.Create(Interface.Menu, "Button", context)

local function InitializeRoleDisplay(self)
	self.iconFrame = Interface.Create("Image", "IconFrame", self)	
	self.iconFrame:SetImage("sml_icon_border.png.dds")
	self.iconFrame:SetSize(32, 32)
	
	self.icon = Interface.Create("Image", "Icon", self)
	self.icon:SetPoint("CENTER", self.iconFrame, "CENTER")
	self.icon:SetLayer(self.iconFrame:GetLayer() - 1)
	
	self.iconBackground = Interface.Create("Image", "Icon", self)
	self.iconBackground:SetPoint("CENTER", self.iconFrame, "CENTER")
	self.iconBackground:SetLayer(self.iconFrame:GetLayer() - 2)
	self.iconBackground:SetBackgroundColor(0,0,0)
	self.iconBackground:SetSize(25, 25)
	
	self.label = Interface.Create("Text", "Label", self)
	self.label:SetFontColor(0.96, 0.87, 0.74, 1)
	self.label:SetFontSize(15)
	self.label:SetEffectGlow({colorR = 0.54, colorG = 0.42, colorB = 0.22, strength = 1.5})
end

local function UpdateRoleLayout(self, tooltip)
	self.label:ClearAll()
	if tooltip then
		self.iconFrame:ClearPoint("CENTERLEFT")
		self.iconFrame:SetPoint("CENTER", self, "CENTER")
		self.label:SetPoint("CENTER", tooltip, "CENTER")
		self.label:SetVisible(false)
	else
		self.iconFrame:ClearPoint("CENTER")
		self.iconFrame:SetPoint("CENTERLEFT", self, "CENTERLEFT", 5, 0)
		self.label:SetPoint("LEFT", self.iconFrame, "RIGHT", 4, nil)
		self.label:SetPoint("CENTERRIGHT", self, "CENTERRIGHT", -4, 0)
		self.label:SetVisible(true)
	end
end

local function OnHighlight(self, highlight)
	if highlight then
		self.iconFrame:SetImage("sml_icon_border_(over)_blue.png.dds")
	else
		self.iconFrame:SetImage("sml_icon_border.png.dds")
	end
end

function menu:ShowBackground(show)
	if show then
		self:SetColor(0, 0, 0, 0.5)
		self:SetHighlightColor(0, 0, 0, 0.3)
		self:SetShadowColor(0, 0, 0, 0.7)
	else
		self:SetColor(0, 0, 0, 0)
		self:SetHighlightColor(0, 0, 0, 0)
		self:SetShadowColor(0, 0, 0, 0)
	end
end

local function InitializeButton(button)
	button:SetHeight(40)
	button:SetBevel(3, 4)
	InitializeRoleDisplay(button:GetFace())
	UpdateRoleLayout(button:GetFace())
	function button.OnHighlight(self, highlight)
		OnHighlight(self:GetFace(), highlight)
		if not (saved.config.background or self:GetPressed()) then
			self:ShowBackground(highlight)
		end
	end
	button:EnableHighlight(true)
end

local function InitializeList(list)
	list:SetBackgroundColor(0, 0, 0, 0.5)
	function list.OnItemHighlight(item, highlight, tooltip)
		if (highlight) then
			item:SetBackgroundColor(0.7, 0.7, 0.7, 0.1)
			if tooltip then
				tooltip:SetWidth(item.label:GetWidth() + 8)
				tooltip:SetHeight(item.label:GetHeight() + 4)
				item.label:SetVisible(true)
			end
		else
			item:SetBackgroundColor(0, 0, 0, 0)
			if tooltip then
				item.label:SetVisible(false)
			end
		end
		OnHighlight(item, highlight)
	end
	list.InitializeItem = InitializeRoleDisplay
	list.UpdateItem = UpdateRoleLayout
end

InitializeButton(menu)
InitializeList(menu:GetList())
Command.Event.Attach(Event.System.Secure.Enter, function() menu:SetActive(false) end, "SystemSecureEnter")
Command.Event.Attach(Event.System.Secure.Leave, function() if not config:IsOpen() then menu:SetActive(true) end end, "SystemSecureLeave")

local icons =
{
	tank = "role_tank_big.png.dds",
	heal = "role_heal_big.png.dds",
	dps = "role_damage_big.png.dds",
	support = "role_support_big.png.dds"
}

local color =
{
	tank = "3366FF",
	heal = "33CC33",
	dps = "FF3333",
	support = "CC33FF",
	none = "999999"
}

local function UpdateRoleDisplay(self, role)
	local customIcon = role:GetCustomIcon()
	if customIcon == "none" then
		self.icon:SetImage(icons[role:GetType()])
		self.icon:SetWidth(40)
		self.icon:SetHeight(40)
	else
		self.icon:SetImage(customIcons.Textures[customIcon])
		self.icon:SetWidth(25)
		self.icon:SetHeight(25)
	end
	
	self.label:SetText(role:GetName())
end

function menu:UpdateItem(index, role)
	local item = self:GetList():GetItem(index)
	UpdateRoleDisplay(item, role)
	item.color = color[role:GetType()]
	for _, button in ipairs({"Left", "Right"}) do
		item:EventMacroSet(Event.UI.Input.Mouse[button].Click, role:GetMacro(button))
	end
end

function menu:UpdateButton(role)
	UpdateRoleDisplay(self:GetFace(), role)
end

function menu.OnItemClick(item, button)
	if not saved.config.background then
		menu:ShowBackground(false)
	end
	if saved.config.notification then
		local message = string.format("%s <font color=\"#%s\">%s</font>", L.addon.notification, item.color, item.label:GetText())
		Utility.Print("system", true, message, true)
	end
end
