--[[
File: Interface/Button.lua
Author: Zormus
--]]

setfenv(1, select(2, ...))

Interface.Button = setmetatable({}, Interface.Frame)
local Button = Interface.Button
Button.__index = Button

Button.type = "Frame"

function Button:Initialize()
	self.border =
	{
		left = Interface.Create("Frame", "Left", self),
		top = Interface.Create("Frame", "Top", self),
		right = Interface.Create("Frame", "Right", self),
		bottom = Interface.Create("Frame", "Bottom", self)
	}
	self.border.left:SetPoint("TOPLEFT", self, "TOPLEFT")
	self.border.top:SetPoint("TOPRIGHT", self, "TOPRIGHT")
	self.border.right:SetPoint("BOTTOMRIGHT", self, "BOTTOMRIGHT")
	self.border.bottom:SetPoint("BOTTOMLEFT", self, "BOTTOMLEFT")
	self.border.left:SetPoint("BOTTOM", self.border.bottom, "TOP")
	self.border.top:SetPoint("LEFT", self.border.left, "RIGHT")
	self.border.right:SetPoint("TOP", self.border.top, "BOTTOM")
	self.border.bottom:SetPoint("RIGHT", self.border.right, "LEFT")
	self.face = Interface.Create("Frame", "Face", self)
	self.face:SetPoint("LEFT", self.border.left, "RIGHT")
	self.face:SetPoint("TOP", self.border.top, "BOTTOM")
	self.face:SetPoint("RIGHT", self.border.right, "LEFT")
	self.face:SetPoint("BOTTOM", self.border.bottom, "TOP")
	self.pressed = false
end

function Button:GetFace()
	return self.face
end

local function SetBevel(self, topLeft, bevel)
	if topLeft then
		self.border.left:SetWidth(bevel)
		self.border.top:SetHeight(bevel)
	else
		self.border.right:SetWidth(bevel)
		self.border.bottom:SetHeight(bevel)
	end
end

function Button:SetBevel(thin, thick)
	SetBevel(self, not self:GetPressed(), thin)
	SetBevel(self, self:GetPressed(), thick)
end

function Button:SetColor(r, g, b, a)
	self.face:SetBackgroundColor(r, g, b, a)
end

local function SetBorderColor(self, topLeft, r, g, b, a)
	if topLeft then
		self.border.left:SetBackgroundColor(r, g, b, a)
		self.border.top:SetBackgroundColor(r, g, b, a)
	else
		self.border.right:SetBackgroundColor(r, g, b, a)
		self.border.bottom:SetBackgroundColor(r, g, b, a)
	end
end

function Button:SetHighlightColor(r, g, b, a)
	SetBorderColor(self, not self:GetPressed(), r, g, b, a)
end

function Button:SetShadowColor(r, g, b, a)
	SetBorderColor(self, self:GetPressed(), r, g, b, a)
end

function Button:GetPressed()
	return self.pressed
end

function Button:Toggle()
	self.pressed = not self.pressed
	self.border.left:SetWidth(self.border.right:GetWidth())
	self.border.right:SetWidth(self.border.top:GetHeight())
	self.border.top:SetHeight(self.border.left:GetWidth())
	self.border.bottom:SetHeight(self.border.right:GetWidth())		
	self.border.left:SetBackgroundColor(self.border.right:GetBackgroundColor())
	self.border.right:SetBackgroundColor(self.border.top:GetBackgroundColor())
	self.border.top:SetBackgroundColor(self.border.left:GetBackgroundColor())
	self.border.bottom:SetBackgroundColor(self.border.right:GetBackgroundColor())
end
