--[[
File: Interface/Menu.lua
Author: Zormus
--]]

setfenv(1, select(2, ...))

Interface.Menu = setmetatable({}, Interface.Button)
local Menu = Interface.Menu
Menu.__index = Menu

function Menu:Initialize()
	Interface.Button.Initialize(self)
	self.OnLeftClick = Menu.Toggle
	self:EnableClick(true, "Left")
	
	self.list = Interface.Create(Interface.List, "List", self:GetParent())
	self.list:SetSecureMode("restricted")
	self.list:SetVisible(false)
	function self.list.OnItemClick(item, button)
		self:Toggle()
		if self.OnItemClick then
			self.OnItemClick(item, button)
		end
	end
	
	self.spacing = 2
	self.fade = 0
end

function Menu:GetList()
	return self.list
end

function Menu:SetLayout(extend)
	if self.extend ~= extend then
		self.extend = extend
		self.list:ClearAll()
		if extend == "Down" then
			self.list:SetPoint("TOPLEFT", self, "BOTTOMLEFT", 0, self.spacing)
			self.list:SetPoint("RIGHT", self, "RIGHT")
		elseif extend == "Up" then
			self.list:SetPoint("BOTTOMLEFT", self, "TOPLEFT", 0, -self.spacing)
			self.list:SetPoint("RIGHT", self, "RIGHT")
		elseif extend == "Right" then
			self.list:SetPoint("TOPLEFT", self, "TOPRIGHT", self.spacing, 0)
			self.list:SetPoint("BOTTOM", self, "BOTTOM")
		elseif extend == "Left" then
			self.list:SetPoint("TOPRIGHT", self, "TOPLEFT", -self.spacing, 0)
			self.list:SetPoint("BOTTOM", self, "BOTTOM")
		end
		self.list:SetOrientation(extend == "Down" or extend == "Up")
	end
end

function Menu:SetFade(fade)
	self.fade = fade / 100
	if self:GetAlpha() ~= 1 then
		self:SetAlpha(self.fade)
	end
end

function Menu:SetActive(active)
	self:EnableClick(active, "Left")
	if active then
		self:SetAlpha(1)
	else
		if self:IsOpen() then
			self:Toggle()
		end
		self:SetAlpha(self.fade)
	end
end

function Menu:IsOpen()
	return self:GetPressed()
end

function Menu:Toggle()
	if not self:IsOpen() and self.OnOpen then
		self:OnOpen()
	end
	Interface.Button.Toggle(self)
	self.list:SetVisible(self:IsOpen())
end

function Menu:SetCount(count)
	self.list:SetCount(count)
end
