--[[
File: Interface/Canvas.lua
Author: Zormus
--]]

setfenv(1, select(2, ...))
Interface.Canvas = setmetatable({}, Interface.Frame)
local Canvas = Interface.Canvas
Canvas.__index = Canvas

Canvas.type = "Frame"

local function UpdateLayout(self)
	local horizontalScale = (self:GetWidth() - self.margins.LEFT - self.margins.RIGHT) / (self.panels.TOPLEFT.image:GetWidth() - self.margins.LEFT - self.margins.RIGHT)
	local verticalScale = (self:GetHeight() - self.margins.TOP - self.margins.BOTTOM) / (self.panels.TOPLEFT.image:GetHeight() - self.margins.TOP - self.margins.BOTTOM)
	local offset =
	{
		LEFT = -self.margins.LEFT * horizontalScale,
		TOP = -self.margins.TOP * verticalScale,
		RIGHT = self.margins.RIGHT * horizontalScale,
		BOTTOM = self.margins.BOTTOM * verticalScale
	}
	for _, x in pairs({"LEFT", "RIGHT", "CENTER"}) do
		for _, y in pairs({"TOP", "BOTTOM", "CENTER"}) do
			local name = y .. x
			if x ~= "CENTER" then
				self.panels[name]:SetWidth(self.margins[x])
			else
				self.panels[name].image:SetPoint("LEFT", self.panels[name], "LEFT", offset.LEFT, nil)
				self.panels[name].image:SetPoint("RIGHT", self.panels[name], "RIGHT", offset.RIGHT, nil)
			end
			if y ~= "CENTER" then
				self.panels[name]:SetHeight(self.margins[y])
			else
				self.panels[name].image:SetPoint("TOP", self.panels[name], "TOP", nil, offset.TOP)
				self.panels[name].image:SetPoint("BOTTOM", self.panels[name], "BOTTOM", nil, offset.BOTTOM)
			end
		end		
	end
end

function Canvas:Initialize()
	self.margins = {LEFT = 0, TOP = 0, RIGHT = 0, BOTTOM = 0}
	self.panels = {}
	for _, x in pairs({"LEFT", "RIGHT", "CENTER"}) do
		for _, y in pairs({"TOP", "BOTTOM", "CENTER"}) do
			local name = y .. x
			self.panels[name] = Interface.Create("Mask", name, self)
			self.panels[name]:SetLayer(-1)
			self.panels[name].image = Interface.Create("Image", "Image", self.panels[name])
			if x ~= "CENTER" then
				self.panels[name]:SetPoint(x, self, x)
				self.panels[name].image:SetPoint(x, self.panels[name], x)
			else
				self.panels[name]:SetPoint("LEFT", self.panels[y .. "LEFT"], "RIGHT")
				self.panels[name]:SetPoint("RIGHT", self.panels[y .. "RIGHT"], "LEFT")
			end
			if y ~= "CENTER" then
				self.panels[name]:SetPoint(y, self, y)
				self.panels[name].image:SetPoint(y, self.panels[name], y)
			else
				self.panels[name]:SetPoint("TOP", self.panels["TOP" .. x], "BOTTOM")
				self.panels[name]:SetPoint("BOTTOM", self.panels["BOTTOM" .. x], "TOP")
			end
		end
	end
	self:EventAttach(Event.UI.Layout.Size, UpdateLayout, "SizeChanged")
end

function Canvas:SetBackgroundImage(image, source)
	local width = self.panels.TOPLEFT.image:GetWidth()
	local height = self.panels.TOPLEFT.image:GetHeight()
	for _, panel in pairs(self.panels) do
		panel.image:SetImage(image, source)
	end
	if self.panels.TOPLEFT.image:GetWidth() ~= width or self.panels.TOPLEFT.image:GetHeight() ~= height then
		UpdateLayout(self)
	end
end

function Canvas:SetBackgroundAlpha(alpha)
	for _, panel in pairs(self.panels) do
		panel.image:SetAlpha(alpha)
	end
end

function Canvas:SetMargins(left, top, right, bottom)
	self.margins.LEFT = left
	self.margins.TOP = top
	self.margins.RIGHT = right
	self.margins.BOTTOM = bottom
	if self.panels then
		UpdateLayout(self)
	end
end
