--[[
File: Interface/Frame.lua
Author: Zormus
--]]

setfenv(1, select(2, ...))
Interface.Frame = {}
local Frame = Interface.Frame
Frame.__index = Frame

function Frame:Initialize()
end

function Frame:SetSize(width, height)
	self:SetWidth(width)
	self:SetHeight(height)
end

function Frame:SaveLocation(location)
	self.location = location
end

function Frame:SetLocation(x, y)
	self:SetPoint("TOPLEFT", self:GetParent(), "TOPLEFT", x, y)
	if self.location then
		self.location.x = self:GetLeft()
		self.location.y = self:GetTop()
	end
end

function Frame:Move(dx, dy)
	self:SetLocation(self:GetLeft() + dx, self:GetTop() + dy)
end

function Frame:Center()
	self:SetPoint("TOPLEFT", self:GetParent(), "CENTER", -self:GetWidth()/2, -self:GetHeight()/2)
	if self.location then
		self.location.x = self:GetLeft()
		self.location.y = self:GetTop()
	end
end

local function GetRegisterEvent(self, enable)
	if enable then
		return self.EventAttach
	else
		return self.EventDetach
	end
end

function Frame:SetDragFrame(frame)
	self.dragFrame = frame
end

local function BeginDrag(self)
	self.drag = Inspect.Mouse()
end

local function EndDrag(self)
	if self.drag then
		self.drag = nil
	end
end

local function Drag(self, _, x, y)
	if self.drag then
		local dragFrame = self.dragFrame or self
		dragFrame:Move(x - self.drag.x, y - self.drag.y)
		self.drag.x = x
		self.drag.y = y
	end
end

function Frame:EnableDrag(enable, button)
	local RegisterEvent = GetRegisterEvent(self, enable)
	RegisterEvent(self, Event.UI.Input.Mouse[button].Down, BeginDrag, "BeginDrag")
	RegisterEvent(self, Event.UI.Input.Mouse[button].Up, EndDrag, "EndDrag")
	RegisterEvent(self, Event.UI.Input.Mouse[button].Upoutside, EndDrag, "EndDrag")
	RegisterEvent(self, Event.UI.Input.Mouse.Cursor.Move, Drag, "Drag")
end

local Click =
{
	Left = function(self) if self.OnLeftClick then self:OnLeftClick() end end,
	Middle = function(self) if self.OnMiddleClick then self:OnMiddleClick() end end,
	Right = function(self) if self.OnRightClick then self:OnRightClick() end end
}

function Frame:EnableClick(enable, button)
	local RegisterEvent = GetRegisterEvent(self, enable)
	RegisterEvent(self, Event.UI.Input.Mouse[button].Click, Click[button], button .. "Click")
end

local function MouseOver(self) if self.OnHighlight then self:OnHighlight(true) end end
local function MouseOut(self) if self.OnHighlight then self:OnHighlight(false) end end

function Frame:EnableHighlight(enable)
	local RegisterEvent = GetRegisterEvent(self, enable)
	RegisterEvent(self, Event.UI.Input.Mouse.Cursor.In, MouseOver, "MouseOver")
	RegisterEvent(self, Event.UI.Input.Mouse.Cursor.Out, MouseOut, "MouseOut")
end
