--[[
File: Interface/RiftIconSelect.lua
Author: Zormus
--]]

setfenv(1, select(2, ...))
Interface.RiftIconSelect = setmetatable({}, Interface.Canvas)
local RiftIconSelect = Interface.RiftIconSelect
RiftIconSelect.__index = RiftIconSelect

function RiftIconSelect:Initialize()
	Interface.Canvas.Initialize(self)
	self:SetHeight(32)
	self:SetMargins(12, 10, 30, 10)
	self:SetBackgroundImage("drop_down_(normal)02.png.dds")
	self:SetLayer(1)
	
	self.icon = Interface.Create("Image", "Icon", self)
	self.icon:SetSize(20, 20)
	self.icon:SetPoint("TOPLEFT", self, "TOPLEFT", 9, 7)
	self.icon:SetImage("role_tank_big.png.dds")
	
	self.text = Interface.Create("Text", "Text", self)
	self.text:SetPoint("TOPLEFT", self, "TOPLEFT", 9 + 25, 7)
	self.text:SetPoint("RIGHT", self, "RIGHT", -29, nil)
	self.text:SetFontSize(13)
	
	self.menu = Interface.Create("Canvas", "Menu", self)
	self.menu:SetVisible(false)
	self.menu:SetPoint("TOPLEFT", self, "BOTTOMLEFT")
	self.menu:SetPoint("RIGHT", self, "RIGHT")
	self.menu:SetMargins(10, 10, 10, 10)
	self.menu:SetBackgroundImage("dropdown_list.png.dds")
	self.menu.items = {}
	self.list = self.menu

	self:EnableHighlight(true)
	self:EnableClick(true, "Left")
	
	self.window = self:GetParent()
	while self.window do
		if self.window.SetActiveSelect then
			break
		end
		self.window = self.window:GetParent()
	end
end

function RiftIconSelect:SetScrollbar(itemCount)
	self.maxVisible = itemCount

	self.scrollarea = Interface.Create("RiftScrollarea", "Scrollarea", self.menu)
	self.scrollarea:SetPoint("TOPLEFT", self.menu, "TOPLEFT")
	self.scrollarea:SetPoint("BOTTOMRIGHT", self.menu, "BOTTOMRIGHT", -5, -8)
	
	self.list = self.scrollarea:GetContent()
end

function RiftIconSelect:OnHighlight(highlight)
	if not self.menu:GetVisible() then
		if highlight then
			self:SetBackgroundImage("drop_down_(over)02.png.dds")
		else
			self:SetBackgroundImage("drop_down_(normal)02.png.dds")
		end
	end
end

function RiftIconSelect:OnLeftClick()
	self.menu:SetVisible(not self.menu:GetVisible())
	if self.menu:GetVisible() then
		self:SetBackgroundImage("drop_down_(click)02.png.dds")
		if self.window then
			self.window:SetActiveSelect(self)
		end
	else
		self:SetBackgroundImage("drop_down_(over)02.png.dds")
		if self.window then
			self.window:SetActiveSelect()
		end
	end
end

function RiftIconSelect:Close()
	if self.menu:GetVisible() then
		self.menu:SetVisible(false)
		self:SetBackgroundImage("drop_down_(normal)02.png.dds")
		if self.window then
			self.window:SetActiveSelect()
		end
	end
end

local function SetSelectedIndex(self, index)
	self.index = index
	self.text:SetText(self.menu.items[index]:GetText())
	local currentSource, currentImage = self.menu.items[index].icon:GetTexture()
	self.icon:SetImage(currentImage)
end

function RiftIconSelect:SetItems(items, reset)
	local itemHeight = 22
	self.menu.count = #items
	
	if not self.maxVisible or self.maxVisible >= self.menu.count then
		self.menu:SetHeight(self.menu.count * itemHeight + 14)
	else
		self.menu:SetHeight(self.maxVisible * itemHeight + 14)
		self.list:SetHeight(self.menu.count * itemHeight + 6)
	end
	
	local function OnHighlight(item, highlight)
		if highlight then
			item:SetBackgroundColor(0.1, 0.1, 0.1, 1)
		else
			item:SetBackgroundColor(0.1, 0.1, 0.1, 0)
		end
	end
	local function OnLeftClick(item)
		local self = item.select
		if self:GetValue() ~= item.value then
			SetSelectedIndex(self, item.index)
			if self.OnChange then
				self:OnChange()
			end
		end
		self:Close()
	end
	
	local offset
	if self.list == self.menu then
		offset = -9
	else
		offset = -2
	end
	for index, item in ipairs(items) do
		if not self.menu.items[index] then
			self.menu.items[index] = Interface.Create("Text", "Item" .. index, self.list)
			
			self.menu.items[index].icon = Interface.Create("Image", "Icon", self.menu.items[index])
			self.menu.items[index].icon:SetSize(itemHeight - 4, itemHeight - 4)
			self.menu.items[index].icon:SetPoint("TOPLEFT", self.list, "TOPLEFT", 9, (index - 1) * itemHeight + 2)
			self.menu.items[index].icon:SetVisible(false)
			
			self.menu.items[index].index = index
			self.menu.items[index].select = self
			self.menu.items[index]:SetPoint("TOPLEFT", self.list, "TOPLEFT", itemHeight + 9, (index - 1) * itemHeight + 4)
			self.menu.items[index]:SetPoint("TOPRIGHT", self.list, "TOPRIGHT", offset, (index - 1) * itemHeight + 4)
			self.menu.items[index]:SetHeight(itemHeight)
			self.menu.items[index]:SetFontSize(13)
			self.menu.items[index]:EnableHighlight(true)
			self.menu.items[index].OnHighlight = OnHighlight
			self.menu.items[index]:EnableClick(true, "Left")
			self.menu.items[index].OnLeftClick = OnLeftClick
		end
		self.menu.items[index].value = item.value
		self.menu.items[index]:SetText(item.label)
		self.menu.items[index]:SetVisible(true)
		
		if item.texture ~= nil then
			self.menu.items[index].icon:SetVisible(false)
			self.menu.items[index].icon:SetImage(item.texture)
		end
	end
	for index = self.menu.count + 1, #self.menu.items do
		self.menu.items[index]:SetVisible(false)
	end
	
	if reset or not self.index or self.index > self.menu.count then
		SetSelectedIndex(self, 1)
	else
		SetSelectedIndex(self, self.index)
	end
end

function RiftIconSelect:GetValue()
	return self.menu.items[self.index].value
end

function RiftIconSelect:SetValue(value)
	if self.menu then
		for index = 1, self.menu.count do
			if self.menu.items[index].value == value then
				SetSelectedIndex(self, index)
				return
			end
		end
	end
end

function RiftIconSelect:SetItemLabel(label, index)
	index = index or self.index
	self.menu.items[index]:SetText(label)
	if index == self.index then
		self.text:SetText(label)
	end
end
