--[[
File: Interface/RiftScrollarea.lua
Author: Zormus
--]]

setfenv(1, select(2, ...))
Interface.RiftScrollarea = setmetatable({}, Interface.Frame)
local RiftScrollarea = Interface.RiftScrollarea
RiftScrollarea.__index = RiftScrollarea

RiftScrollarea.type = "Frame"

local function UpdateScrollbar(self)
	local range = math.max(self.content:GetHeight() - self.mask:GetHeight(), 0)
	self.scrollbar:SetRange(0, range)
	self.scrollbar:SetThickness(self.mask:GetHeight())
	self.scrollbar:SetEnabled(range > 0)
end

local function ContentSizeChanged(content)
	UpdateScrollbar(content:GetParent():GetParent())
end

local function UpdateContentPosition(scrollbar)
	local self = scrollbar:GetParent()
	self.content:SetPoint("TOP", self.mask, "TOP", nil, -scrollbar:GetPosition())
end

function RiftScrollarea:Initialize()
	self.scrollbar = Interface.Create("RiftScrollbar", "Scrollbar", self)
	self.scrollbar:SetPoint("TOPRIGHT", self, "TOPRIGHT")
	self.scrollbar:SetPoint("BOTTOM", self, "BOTTOM")
	self.scrollbar:SetRange(0, 0)
	self.scrollbar:SetPosition(0)
	
	self.mask = Interface.Create("Mask", "Mask", self)
	self.mask:SetPoint("TOPLEFT", self, "TOPLEFT")
	self.mask:SetPoint("BOTTOM", self, "BOTTOM")
	self.mask:SetPoint("RIGHT", self.scrollbar, "LEFT")
	
	self.content = Interface.Create("Frame", "Content", self.mask)
	self.content:SetPoint("TOPLEFT", self.mask, "TOPLEFT")
	self.content:SetPoint("RIGHT", self.mask, "RIGHT")

	UpdateScrollbar(self)
	self:EventAttach(Event.UI.Layout.Size, UpdateScrollbar, "SizeChanged")
	self.content:EventAttach(Event.UI.Layout.Size, ContentSizeChanged, "SizeChanged")
	self.scrollbar:EventAttach(Event.UI.Scrollbar.Change, UpdateContentPosition, "ScrollbarChanged")
end

function RiftScrollarea:GetContent()
	return self.content
end
