--[[
File: Interface/RiftTabview.lua
Author: Zormus
--]]

setfenv(1, select(2, ...))
Interface.RiftTabview = setmetatable({}, Interface.Frame)
local RiftTabview = Interface.RiftTabview
RiftTabview.__index = RiftTabview

RiftTabview.type = "Frame"

function RiftTabview:Initialize()
	self.tabs = {}
	self.panel = Interface.Create("Canvas", "Panel", self)
	self.panel:SetMargins(12, 12, 12, 12)
	self.panel:SetBackgroundImage("KeyCell_(normal).png.dds")
	self.panel:SetBackgroundAlpha(0.7)
	self.panel:SetPoint("BOTTOMLEFT", self, "BOTTOMLEFT")
	self.panel:SetPoint("BOTTOMRIGHT", self, "BOTTOMRIGHT")
	self.panel:SetPoint("TOP", self, "TOP", nil, 38)
	self:EventAttach(Event.UI.Layout.Size, UpdateTabWidth, "SizeChanged")
end

function UpdateTabWidth(self)
	local width = self.panel:GetWidth() / #self.tabs
	for index, tab in ipairs(self.tabs) do
		tab.button:SetWidth(width)
	end
end

local function ActivateTab(self)
	self:GetParent():SetActiveTab(self)
end

function RiftTabview:AddTab(name)
	local tab = {name = name}
	table.insert(self.tabs, tab)
	tab.button = Interface.Create("Canvas", "Button" .. #self.tabs, self)
	tab.button:SetPoint("TOP", self, "TOP")
	tab.button:SetPoint("BOTTOM", self.panel, "TOP", nil, 8)
	if #self.tabs == 1 then
		tab.button:SetPoint("LEFT", self.panel, "LEFT")
	else
		tab.button:SetPoint("LEFT", self.tabs[#self.tabs - 1].button, "RIGHT")
	end
	UpdateTabWidth(self)
	tab.button:SetMargins(12, 12, 12, 12)
	tab.button:SetBackgroundImage("KeyCell_(normal).png.dds")
	tab.button:SetBackgroundAlpha(0.7)
	tab.button.label = Interface.Create("Text", "Label", tab.button)
	tab.button.label:SetFontColor(0.96, 0.87, 0.74, 1)
	tab.button.label:SetFontSize(15)
	tab.button.label:SetEffectGlow({colorR = 0.54, colorG = 0.42, colorB = 0.22, strength = 1.5})
	tab.button.label:SetText(name)
	tab.button.label:SetPoint("CENTER", tab.button, "CENTER")
	tab.button.OnLeftClick = ActivateTab
	tab.button:EnableClick(true, "Left")
	if #self.tabs == 1 then
		self:SetActiveTab(tab.button)
	else
		tab.button:SetAlpha(0.5)
	end
end

function RiftTabview:SetActiveTab(button)
	for index, tab in ipairs(self.tabs) do
		if tab.button == button then
			tab.button:SetAlpha(1)
			if not tab.panel then
				tab.panel = Interface.Create("Frame", "Panel" .. index, self.panel)
				tab.panel:SetPoint("TOPLEFT", self.panel, "TOPLEFT", 20, 16)
				tab.panel:SetPoint("BOTTOMRIGHT", self.panel, "BOTTOMRIGHT", -20, -16)
				if self.OnInitializeTab then
					self.OnInitializeTab(tab.panel, tab.name)
				end
			end
			if tab.panel.OnShow then
				tab.panel.OnShow()
			end
			tab.panel:SetVisible(true)
			self.activeTab = tab.name
		else
			tab.button:SetAlpha(0.5)
			if tab.panel and tab.panel:GetVisible() then
				tab.panel:SetVisible(false)
				if tab.panel.OnHide then
					tab.panel.OnHide()
				end
			end
		end
	end
end

function RiftTabview:GetActiveTab()
	return self.activeTab
end
