--[[
File: Interface/RiftWindow.lua
Author: Zormus
--]]

setfenv(1, select(2, ...))
Interface.RiftWindow = setmetatable({}, Interface.Frame)
local RiftWindow = Interface.RiftWindow
RiftWindow.__index = RiftWindow

local function ClearTextfieldFocus(self)
	if self.textfield then
		self.textfield:SetKeyFocus(false)
		self.textfield = nil
	end
end
	
local function CloseActiveSelect(self)
	if self.select then
		self.select:Close()
	end
end

local function OnClick(self, element)
	if element:GetKeyFocus() then
		self.textfield = element
	else
		ClearTextfieldFocus(self)
	end
	if self.selectOpen then
		self.selectOpen = false
	else
		local scrollbar
		if self.select and Utility.Type(element) == "RiftScrollbar" then
			scrollbar = element:GetParent()
			while scrollbar do
				if scrollbar == self.select then
					break
				end
				scrollbar = scrollbar:GetParent()
			end
		end
		if not scrollbar then
			CloseActiveSelect(self)
		end
	end
end

function RiftWindow:Initialize()
	self.dragBar = Interface.Create("Frame", "DragBar", self)
	self.dragBar:SetPoint("TOPLEFT", self, "TOPLEFT", 44, 12)
	self.dragBar:SetPoint("RIGHT", self, "RIGHT", -44, nil)
	self.dragBar:SetHeight(34)
	self.dragBar:SetDragFrame(self)
	self.dragBar:EnableDrag(true, "Left")
	
	self.close = Interface.Create("RiftButton", "CloseButton", self)
	self.close:SetSkin("close")
	self.close:SetPoint("TOPRIGHT", self, "TOPRIGHT", -8, 16)
	self.close:EventAttach(Event.UI.Input.Mouse.Left.Click, function(close) close:GetParent():Close() end, "Close")
	
	for _, button in ipairs({"Left", "Right"}) do
		self:EventAttach(Event.UI.Input.Mouse[button].Click.Bubble, function(element) OnClick(self, element) end, "OnClick")
	end
end

function RiftWindow:Close()
	ClearTextfieldFocus(self)
	CloseActiveSelect(self)
	if self.OnClose then
		self:OnClose()
	end
	self:SetVisible(false)
end

function RiftWindow:SetActiveSelect(select)
	CloseActiveSelect(self)
	self.select = select
	if select then
		self.selectOpen = true
	end
end
