--[[
File: Interface/Textfield.lua
Author: Zormus
--]]

setfenv(1, select(2, ...))
Interface.Textfield = setmetatable({}, Interface.Frame)
local Textfield = Interface.Textfield
Textfield.__index = Textfield

Textfield.type = "RiftTextfield"

local function OnTextChange(self)
	if self.maxLines then
		local lines = 1
		local text = self:GetText()
		local position = string.find(text, "\r")
		while position do
			lines = lines + 1
			if lines > self.maxLines then
				self:SetText(string.sub(text, 1, position - 1))
				break
			end
			position = string.find(text, "\r", position + 1)
		end
	end
	if self.OnChange then
		self.OnChange()
	end
end

local function OnKeyType(self, _, key)
	if key == "\r" then
		if self.maxLines == 1 then
			self:SetKeyFocus(false)
		else
			local text = self:GetText()
			local cursor = self:GetCursor()
			self:SetText(string.format("%s\r%s", string.utf8sub(text, 1, cursor), string.utf8sub(text, cursor + 1)))
			self:SetCursor(cursor + 1)
			OnTextChange(self)
		end
	end
end

function Textfield:Initialize()
	self:EventAttach(Event.UI.Input.Key.Type, OnKeyType, "KeyType")
	self:EventAttach(Event.UI.Textfield.Change, OnTextChange, "TextChange")
end

function Textfield:SetMaximumLines(maximum)
	self.maxLines = maximum
end

function Textfield:GetLines()
	local text = string.trim(self:GetText())
	local lines = {}
	while string.len(text) > 0 do
		local line
		local position = string.find(text, "\r")
		if position then
			line = string.sub(text, 1, position - 1)
			text = string.sub(text, position + 1)
		else
			line = text
			text = ""
		end
		line = string.trim(line)
		if string.len(line) > 0 then
			table.insert(lines, line)
		end
	end
	return lines
end

function Textfield:SetLines(lines)
	local count = #lines
	if self.maxLines then
		count = math.min(count, self.maxLines)
	end
	if count > 0 then
		local text
		for i = 1, count do
			if i == 1 then
				text = lines[i]
			else
				text = string.format("%s\r%s", text, lines[i])
			end
		end
		self:SetText(text)
	else
		self:SetText("")
	end
end
