--[[
File: Timer.lua
Author: Zormus
--]]

setfenv(1, select(2, ...))
Timer = {}
Timer.__index = Timer

local function Callback(self)
	if not self.time or Inspect.Time.Real() > self.time then
		if self.OnElapsed() then
			self:Stop()
		elseif self.time then
			if self.interval then
				self.time = self.time + self.interval
			else
				self.time = nil
			end
		end
	end
end

function Timer.Create(interval)
	local timer = setmetatable({}, Timer)
	timer.interval = interval
	timer.Callback = function() Callback(timer) end
	return timer
end

function Timer:Start(delay)
	if self.OnElapsed then
		delay = delay or 0
		self.time = Inspect.Time.Real() + delay
		Command.Event.Attach(Event.System.Update.Begin, self.Callback, "Event.System.Update.Begin")
	end
end

function Timer:Stop()
	Command.Event.Detach(Event.System.Update.Begin, self.Callback)
end
