--[[
File: Role.lua
Author: Zormus
--]]

setfenv(1, select(2, ...))

Role = {}
Role.__index = Role

local list
local id = {}
local function GetRoleName(slot)
	if not id[slot] then
		id[slot] = string.format("h%016X", slot - 1)
	end
	return list[id[slot]]
end

function UpdateRoleList()
	local roleList = Inspect.Role.List()
	if next(roleList) ~= nil then
		list = roleList
		local slot = #roles + 1
		while GetRoleName(slot) do
			roles[slot] = Role.Create(slot)
			slot = slot + 1
		end
	end
end

function Role.Create(slot)
	local role = setmetatable({}, Role)
	role.slot = slot
	saved.roles[slot] = saved.roles[slot] or {}
	role.data = saved.roles[slot]
	
	local defaultData =
	{
		menu = true,
		type = "none",
		customIcon = "none",
		Left =
		{
			equipment = nil,
			items = {},
			wardrobe = nil
		},
		Right =
		{
			equipment = nil,
			items = {},
			wardrobe = nil
		},
		custom = {},
		kAlert =
		{
			set = nil,
			subset = nil
		}
	}
	Utility.SetDefaultValues(role.data, defaultData)
	
	-- Convert saved variables from pre-1.9 versions
	for _, button in ipairs({"Left", "Right"}) do
		if role.data[button].equip then
			role.data[button].equipment = role.data[button].equip
			role.data[button].equip = nil
		end
	end
	
	role.macros = {}
	role:UpdateMacros()
	return role
end

function Role:GetMenu()
	return self.data.menu
end

function Role:SetMenu(menu)
	self.data.menu = menu
end

function Role:GetSlot()
	return self.slot
end

function Role:GetName()
	return GetRoleName(self.slot)
end

function Role:GetType()
	return self.data.type
end

function Role:SetType(type)
	self.data.type = type
end

function Role:GetCustomIcon()
	return self.data.customIcon
end

function Role:SetCustomIcon(customIcon)
	self.data.customIcon = customIcon
end

function Role:GetEquipment(button)
	return self.data[button].equipment
end

function Role:SetEquipment(button, set)
	self.data[button].equipment = set
end

function Role:GetItems(button)
	return self.data[button].items
end

function Role:SetItems(button, items)
	self.data[button].items = items
end

function Role:GetWardrobe(button)
	return self.data[button].wardrobe
end

function Role:SetWardrobe(button, set)
	self.data[button].wardrobe = set
end

function Role:GetCustom()
	return self.data.custom
end

function Role:SetCustom(custom)
	self.data.custom = custom
end

function Role:GetAlertSet()
	return self.data.kAlert.set
end

function Role:SetAlertSet(set)
	self.data.kAlert.set = set
end

function Role:GetAlertSubset()
	return self.data.kAlert.subset
end

function Role:SetAlertSubset(subset)
	self.data.kAlert.subset = subset
end

function Role:GetMacro(button)
	return self.macros[button] or self.macros["Left"]
end

function Role:UpdateMacros()
	for _, button in ipairs({"Left", "Right"}) do
		local macro = string.format("role %i", self:GetSlot())

		if self:GetWardrobe(button) then
			macro = string.format("%s\nwardrobe %i", macro, self:GetWardrobe(button))
		end

		if self:GetEquipment(button) then
			macro = string.format("%s\nloadequip %i\nwait 1", macro, self:GetEquipment(button))
			for _, item in ipairs(self:GetItems(button)) do
				macro = string.format("%s\nequip %s", macro, item)
			end
		else
			macro = string.format("%s\nwait 1", macro)
		end
		
		for _, custom in ipairs(self:GetCustom()) do
			macro = string.format("%s\n%s", macro, custom)
		end
		
		if KaruulAlert() then
			if self:GetAlertSet() then
				macro = string.format("%s\nkalert set=%s", macro, self:GetAlertSet())
			end
			if self:GetAlertSubset() then
				macro = string.format("%s\nkalert subset=%s", macro, self:GetAlertSubset())
			end
		end

		self.macros[button] = macro
	end
end

Compare = {}

function Compare.Slot(role1, role2)
	return role1:GetSlot() < role2:GetSlot()
end

function Compare.Name(role1, role2)
	return string.lower(role1:GetName()) < string.lower(role2:GetName())
end

function Compare.Type(role1, role2)
	if role1:GetType() ~= role2:GetType() then
		local order = {tank = 1, heal = 2, dps = 3, support = 4, none = 5}
		return order[role1:GetType()] < order[role2:GetType()]
	else
		return Compare.Name(role1, role2)
	end
end
